#Region "Disclaimer"
' By Teo Lachev "Applied Microsoft Analysis Services 2005"
'============================================================================
'   File:       StoredProcedures.vb
'   Summary:    SSAS Stored procedure examples
'   License:    This code is covered by the Apache license. 
'               For more information read EULA.txt
'--------------------------------------------------------------------
''===========================================================================
#End Region
Imports Microsoft.AnalysisServices
Imports Microsoft.AnalysisServices.AdomdServer
Imports System.Diagnostics

Public Class StoredProcedures
    ' ------------------------------------------------------------------------------
    ' Calculates the percentage difference between two decimal values
    ' Call example
    '   CREATE MEMBER CURRENTCUBE.[MEASURES].[PctDiff]
    '   AS AdventureWorks!PctDiff([Measures].[Reseller Sales Amount],
    '        [Measures].[Internet Sales Amount]), 
    '   FORMAT_STRING = "Percent", 
    '   VISIBLE = 1  ;   
    '-------------------------------------------------------------------------------
    Public Shared Function PctDiff(ByVal x As Decimal, ByVal y As Decimal) As Object

        If y = 0 Then
            Return "NA"
        Else
            Return (x - y) / y
        End If

    End Function
    ' ------------------------------------------------------------------------------
    ' Returns the current day
    ' Call example
    ' SELECT {[Internet Sales Amount], [Sales Profit], [Average Profit]} on 0,
    '		StrToMember (AdventureWorks!GetCurrentDay()) on 1
    ' FROM [SOS OLAP]  
    '-------------------------------------------------------------------------------
    Public Shared Function GetCurrentDay() As String
        Return "[Date].[Calendar Date].[2003].[H" + CStr(IIf(Month(Now()) <= 6, 1, 2)) + _
                " CY 2003].[Q" + CStr(DatePart("q", Now())) + " CY 2003].[" + Format(Now(), "MMMM") + "].[" _
                + Format(Now(), "MMMM") + " " + CStr(Day(Now())) + "]"
    End Function
    ' ------------------------------------------------------------------------------
    ' Outputs the fully qualified member names of a set
    ' Call Example
    ' 1. Install DebugView from sysinternals.com and open it.
    ' 2. Use the SQL Server Management Studio to fire an MDX query that returns a set, e.g.
    ' 	SELECT AdventureWorks.ShowSet(
    '	SetToStr( 
    '			{TopCount([Reseller].[Reseller].[Reseller].Members, 5, 
    '			[Reseller Profit Dollar Amount])} 
    '			)
    '            ) ON COLUMNS
    '   FROM [SOS OLAP]	
    '-------------------------------------------------------------------------------
    <CLSCompliant(False)> _
    Public Shared Function ShowSet(ByVal inputSet As String) As AdomdServer.Set
        Trace.WriteLine(inputSet)
        Return AdomdServer.MDX.StrToSet(inputSet)
    End Function
    ' ------------------------------------------------------------------------------
    ' Filter a set given an expression
    ' Disclaimer -- The C# version of this stored procedure can be found in the AmoAdventureWorks
    '               code sample which is installed with Analysis Services
    '
    ' Call Example
    ' SELECT [Measures].[Internet Sales Amount] ON 0,
    ' AdventureWorks.FilterSet( [Customer].Customer.Members, "NOT IsEmpty([Measures].[Internet Sales Amount])")  on 1
    ' FROM [SOS OLAP]	
    '
    '  or
    ' Select {[Internet Sales Amount], [Sales Profit], [Average Profit]} on 0,
    ' AdventureWorks.FilterSet([Customer].[Customers by Geography].[City].Members,
    ' "[Measures].[Internet Sales Amount] > 250000") on 1
    ' From [SOS OLAP]
    '-------------------------------------------------------------------------------
    <CLSCompliant(False)> _
    Public Shared Function FilterSet(ByVal inputSet As AdomdServer.Set, ByVal filterExpression As String) As AdomdServer.Set

        Dim tuple As AdomdServer.Tuple
        Dim expr As AdomdServer.Expression = New AdomdServer.Expression(filterExpression)

        Dim resultSetBuilder As AdomdServer.SetBuilder = New AdomdServer.SetBuilder()

        For Each tuple In inputSet.Tuples
            If CType(expr.Calculate(tuple), Boolean) Then
                resultSetBuilder.Add(tuple)
            End If
        Next

        Return resultSetBuilder.ToSet()
    End Function
    ' ------------------------------------------------------------------------------
    ' Evaluates the values of all tuples in a set and filters those that exceeds the specified ceiling
    ' Call Example
    ' Select {[Internet Sales Amount], [Reseller Sales Amount], [Sales Profit], [Average Profit]} on 0,
    ' AdventureWorks.FilterSetByCeiling(([Customer].[Customers by Geography].[City].Members, 
    ' [Product].[Product by Category].[Subcategory].&[1]), "Measures.[Internet Sales Amount]", 20000) on 1
    ' From [SOS OLAP]
    '-------------------------------------------------------------------------------
    <CLSCompliant(False)> _
    Public Shared Function FilterSetByCeiling(ByVal inputSet As AdomdServer.Set, _
                        ByVal measure As String, ByVal ceiling As Integer) As [Set]

        Dim tuple As Tuple
        Dim expr As AdomdServer.Expression = New Expression()
        Dim resultSetBuilder As SetBuilder = New SetBuilder()
        Dim context, expText As String
        Dim mdxValue As MDXValue

        ' Enumerate the tuples in the set
        For Each tuple In inputSet.Tuples
            ' Get the tuple members
            context = GetCurrentContextFromTuple(tuple)
            ' Prepare the expression
            expText = String.Format("({0} {1})", context, measure)
            expr.ExpressionText = expText
            ' Get the value
            mdxValue = expr.Calculate(tuple)

            If mdxValue.ToInt32() <= ceiling Then  ' ! assume the cell value will be int
                resultSetBuilder.Add(tuple)
            End If
        Next

        Return resultSetBuilder.ToSet()
    End Function

    Private Shared Function GetCurrentContextFromTuple(ByVal tuple As Tuple) As String

        Dim context As String = String.Empty
        Dim member As Member

        For Each member In tuple.Members
            context += member.UniqueName + ", "
        Next

        Return context

    End Function

End Class

