Imports System.IO
Imports System.Xml
Imports System.Data
Imports System.Configuration
Imports Microsoft.Office.Interop.Owc11
Imports AxMicrosoft.Office.Interop.Owc11
Imports Microsoft.AnalysisServices.AdomdClient

' Coordinate Types
Enum CoordinateType
    Cube = 1
    Dimension
    Level
    Member
    [Set]
    Cell
End Enum
Public Class Report

    Private m_connectionString As String = ConfigurationManager.AppSettings("connectionString")
    Private Sub Report_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ConfigureOWC()
        LoadUserSettings()
    End Sub

    Private Sub ConfigureOWC()
        pivotTable.ConnectionString = m_connectionString
        pivotTable.Refresh()
    End Sub
    ''' <summary>
    ''' Retrieves the actions defined for a given coordinate
    ''' </summary>
    ''' <param name="coordinate">The coordinate text, e.g. ([Measures].[Reseller Sales Amount],[Reseller].[Reseller].&amp;[17],[Date].[Date].&amp;[1])</param>
    ''' <param name="coordinateType">The coordinate type. Should be always 6 for writeback</param>
    ''' <returns>A datatable with the actions found</returns>
    ''' <remarks>Remember to enclose the coordinate with ()</remarks>
    Private Function GetActions(ByVal coordinate As String, ByVal coordinateType As CoordinateType) As DataTable
        Dim connection As New AdomdConnection(m_connectionString)
        Dim actions As DataSet
        Dim action As DataRow
        Dim i As Integer
        Dim restrictions As New AdomdRestrictionCollection()

        ' This is what the MDSCHEMA_ACTIONS call should look like
        '<CATALOG_NAME>SOS OLAP</CATALOG_NAME>
        '<CUBE_NAME>SOS OLAP</CUBE_NAME>
        '<COORDINATE>([Measures].[Finance Amount], [Date].[Date].&[1])</COORDINATE>
        '<COORDINATE_TYPE>6</COORDINATE_TYPE> -- 6 is cell-level action

        restrictions.Add(New AdomdRestriction("CATALOG_NAME", "SOS OLAP"))
        restrictions.Add(New AdomdRestriction("CUBE_NAME", "SOS OLAP"))
        restrictions.Add(New AdomdRestriction("COORDINATE", coordinate))
        restrictions.Add(New AdomdRestriction("COORDINATE_TYPE", Int32.Parse(coordinateType)))

        ' Open the connection
        connection.Open()
        actions = connection.GetSchemaDataSet("MDSCHEMA_ACTIONS", restrictions)

        ' Output all returned actions for tracing purposes
        Trace.WriteLine(String.Empty)
        If actions.Tables(0).Rows.Count > 0 Then
            For Each action In actions.Tables(0).Rows
                For i = 0 To action.Table.Columns.Count - 1
                    Trace.WriteLine(String.Format("Property: {0}, Value: {1}", action.Table.Columns(i).Caption, action(i).ToString()))
                Next
                Trace.WriteLine("--------------------------")
            Next
        End If

        ' Return the action table
        Return actions.Tables(0)
    End Function

    ''' <summary>
    ''' BeforeContextMenu PivotTable handler
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks>Fires before the context menu is expanded</remarks>
    Private Sub pivotTable_BeforeContextMenu(ByVal sender As System.Object, ByVal e As AxMicrosoft.Office.Interop.Owc11.IPivotControlEvents_BeforeContextMenuEvent) Handles pivotTable.BeforeContextMenu
        Dim newMenu As Object() = Nothing
        Dim coorType As Integer
        Dim coorValue As String = Nothing
        Dim coorCaption As String = Nothing
        Dim actions As DataTable
        Dim aggregates As PivotAggregates = Nothing
        Dim aggregate As PivotAggregate = Nothing
        Dim actionCount As Integer
        Dim i As Integer

        If pivotTable.Selection Is Nothing Then
            Return
        End If

        If pivotTable.Selection.Count <> 1 Then
            Return
        End If

        ' Determine the selection type
        ' We handle only PivotAggregates (cell)
        If pivotTable.SelectionType = "PivotAggregates" Then
            coorType = CoordinateType.Cell
            coorCaption = pivotTable.Selection(0).Total.Caption
            aggregates = CType(pivotTable.Selection, PivotAggregates)
            aggregate = aggregates(0)
            ' Translate the selected cell to coordinates
            coorValue = GetCoordinateFromCell(aggregate.Cell)
            ' Add the measure coordinate
            coorValue = " (" + pivotTable.Selection(0).Total.UniqueName + IIf(coorValue Is Nothing, String.Empty, "," + coorValue) + ")"
        Else
            Return
        End If


        ' Get the actions for this coordinate
        actions = GetActions(coorValue, coorType)
        actionCount = actions.Select("ACTION_NAME='Writeback'").Length

        ' If there are any actions defined, we need to overwrite the menu
        If actionCount > 0 Then
            Dim menu As Object() = CType(e.menu.Value, Object())

            newMenu = New Object(menu.Length + actionCount) {}
            menu.CopyTo(newMenu, 0)
            newMenu(menu.Length) = Nothing  ' placeholder
            For i = 0 To actionCount - 1
                newMenu(menu.Length + 1 + i) = New Object() {actions.Rows(i)("ACTION_NAME"), coorValue}
            Next
            e.menu.Value = newMenu
        End If

    End Sub
    ''' <summary>
    ''' Helper function to translate a pivot cell to coordinate
    ''' </summary>
    ''' <param name="cell"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetCoordinateFromCell(ByVal cell As PivotCell)

        Dim coor As String = Nothing

        ' Column coordinate
        If cell.ColumnMember.Field Is Nothing = False Then
            coor = cell.ColumnMember.UniqueName
        End If
        ' Row coordinate
        If cell.RowMember.Field Is Nothing = False Then
            coor += IIf(coor Is Nothing, String.Empty, ",") + cell.RowMember.UniqueName
        End If

        Return coor

    End Function
    ''' <summary>
    ''' CommandExecute fires when a context menu item is selected 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub pivotTable_CommandExecute(ByVal sender As System.Object, ByVal e As AxMicrosoft.Office.Interop.Owc11.IPivotControlEvents_CommandExecuteEvent) Handles pivotTable.CommandExecute
        ' Need to determine if this is our menu item
        If TypeOf e.command Is String Then
            ' get the action value which is the tuple in our case
            Dim tuple As String = e.command.ToString()
            Dim result As String = InputBox(String.Format("Enter value for tuple {0}.", tuple), "Writeback")
            If result <> String.Empty Then
                Try
                    DoWriteback(tuple, Double.Parse(result))
                    MsgBox("Writeback changes are commited successfully.", MsgBoxStyle.Information)
                Catch ex As Exception
                    MsgBox(ex.ToString(), MsgBoxStyle.Critical)
                End Try
            End If
        End If
    End Sub
    ''' <summary>
    ''' Performs writeback to a tuple
    ''' </summary>
    ''' <param name="tuple">The tuple coordinates</param>
    ''' <param name="value">The value</param>
    ''' <remarks></remarks>
    Private Sub DoWriteback(ByVal tuple As String, ByVal value As Double)

        Dim command As New AdomdCommand()
        Dim connection As New AdomdConnection(ConfigurationManager.AppSettings("connectionString"))
        Dim transaction As AdomdTransaction = Nothing

        Try
            connection.Open()
            command.Connection = connection
            ' Update statement require transaction because changes are not immediately written to the database
            transaction = connection.BeginTransaction()
            ' Writeback requires UPDATE CUBE statement
            command.CommandText = String.Format("UPDATE CUBE [SOS OLAP] SET {0} = {1}", tuple, value)
            ' Fire the query
            command.ExecuteNonQuery()
            ' If the following line completes successfully, the changes are written to the relational table
            transaction.Commit()
        Catch ex As Exception
            If transaction Is Nothing = False Then
                transaction.Rollback()
            End If
            Throw ex
        Finally
            If connection Is Nothing = False Then
                connection.Close()
            End If
        End Try
    End Sub

    '/ <summary>
    '/ Gets the user preferences from the isolated storage
    '/ </summary>
    Sub LoadUserSettings()
        Try
            Dim dom As New XmlDocument()
            Dim s As Stream = Util.GetSettingsStream(Me.Name, FileMode.Open, FileAccess.Read, FileShare.Read)
            dom.Load(s)
            pivotTable.XMLData = dom.SelectSingleNode("//xmlData").InnerText
        Catch ' no big deal if we fail - this is a convenience feature only
        End Try
    End Sub 'LoadUserSettings
    '/ <summary>
    '/ Saves the user preferences to the isolated storage
    '/ </summary>
    Sub SaveUserSettings()
        Try
            Dim s As Stream = Util.GetSettingsStream(Me.Name, FileMode.Create, FileAccess.ReadWrite, FileShare.None)
            Dim w As New XmlTextWriter(s, System.Text.Encoding.UTF8)
            w.WriteStartElement(Me.Name)
            w.WriteElementString("xmlData", pivotTable.XMLData)
            w.WriteEndElement()
            w.Close()
        Catch ' no big deal if we fail - this is a convenience feature only
        End Try
    End Sub 'SaveUserSettings
    Private Sub Writeback_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        SaveUserSettings()
    End Sub
End Class
