Imports System.IO
Imports System.Xml
Imports Microsoft.AnalysisServices
Imports ADOMD = Microsoft.AnalysisServices.AdomdClient
Public Class Main

    Dim m_server As New Server

    Private Enum Icons As Integer
        Dimension = 0
        DisplayFolder
        Hierarchy
        AttributeHierarchy
        Level
    End Enum


    Private Sub btnConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnConnect.Click
        ClearForm()
        Connect()   ' Connect to the database
        LoadCubes() ' Load cubes
    End Sub
    ''' <summary>
    ''' Connects to the specified SSAS database
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub Connect()
        Me.Cursor = Cursors.WaitCursor
        Try
            Disconnect()
            Try
                m_server.Connect("Data Source=" & txtServer.Text)
            Catch ex As Exception
                MessageBox.Show(ex.Message)
                Return
            End Try
        Finally
            Me.Cursor = Cursors.Default
        End Try
    End Sub 'Connect
    ''' <summary>
    ''' Load the cubes in the database
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadCubes()
        Dim cube As Cube

        cmbCubes.Items.Clear()

        'Retrieve list of cubes
        Dim cubes As CubeCollection = m_server.Databases.GetByName(txtDatabase.Text).Cubes

        For Each cube In cubes
            Dim listItem As New ListItem(cube.Name, cube)
            cmbCubes.Items.Add(listItem)
        Next

    End Sub
    ''' <summary>
    ''' Load the cube dimensions and multi-level hierarchies
    ''' </summary>
    ''' <param name="cube"></param>
    ''' <remarks></remarks>
    Private Sub LoadMetadata(ByVal cube As Cube)
        Dim parentNode As TreeNode
        Dim tempNode As TreeNode

        treeMeta.SuspendLayout()
        treeMeta.Nodes.Clear()

        ' Display Measures
        Dim measureGroup As MeasureGroup
        Dim measure As Measure

        For Each measureGroup In cube.MeasureGroups
            parentNode = New TreeNode(measureGroup.Name, Icons.DisplayFolder, Icons.DisplayFolder)
            treeMeta.Nodes.Add(parentNode)
            For Each measure In measureGroup.Measures
                tempNode = New TreeNode(measure.Name, Icons.AttributeHierarchy, Icons.AttributeHierarchy)
                tempNode.Tag = measure
                parentNode.Nodes.Add(tempNode)
            Next
        Next

        ' Display Dimensions
        parentNode = New TreeNode("Dimensions", Icons.DisplayFolder, Icons.DisplayFolder)
        treeMeta.Nodes.Add(parentNode)

        Dim dimension As CubeDimension

        For Each dimension In cube.Dimensions
            tempNode = New TreeNode(dimension.Name, Icons.Dimension, Icons.Dimension)
            tempNode.Tag = dimension

            parentNode.Nodes.Add(tempNode)

        Next dimension

        treeMeta.ResumeLayout()
    End Sub
    ''' <summary>
    ''' Generates the Local Cube Query
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub GenerateQuery()
        Dim listItem As ListItem = CType(cmbCubes.SelectedItem, ListItem)
        Dim cube As Cube = CType(listItem.Value, Cube)
        Dim dimensions As String = String.Empty
        Dim measures As String = String.Empty
        Dim array As String()
        Dim iterator As String

        btnGenerate.Enabled = txtDimensions.Text.Trim() <> String.Empty And txtMeasures.Text.Trim() <> String.Empty
        ' Get measures
        array = txtMeasures.Text.Split(",")
        If array(0) <> String.Empty Then
            For Each iterator In array
                measures += "," + String.Format("MEASURE [{0}].{1}", cube.Name, iterator)
            Next
            measures = measures.Remove(0, 1) ' remove trailing comma
        End If

        ' Get dimensions
        array = txtDimensions.Text.Split(",")
        If array(0) <> String.Empty Then
            For Each iterator In array
                dimensions += "," + String.Format("DIMENSION [{0}].{1}", cube.Name, iterator.Trim())
            Next
            dimensions = dimensions.Remove(0, 1) ' remove trailing comma
        End If

        txtQuery.Text = String.Format("CREATE GLOBAL CUBE Sales STORAGE '{0}\Sales.cub' FROM [{1}] ({2}, {3})", My.Computer.FileSystem.SpecialDirectories.MyDocuments, cube.Name, measures, dimensions)
    End Sub
    ''' <summary>
    ''' Creates the local cube
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub CreateLocalCube()
        ' Use ADOMD.NET to send the statement 
        Dim connection As New ADOMD.AdomdConnection(String.Format("Data Source={0};Initial Catalog={1};", txtServer.Text, txtDatabase.Text))
        Dim command As New ADOMD.AdomdCommand(txtQuery.Text)

        Me.Cursor = Cursors.WaitCursor
        Try
            connection.Open()
            command.Connection = connection
            command.ExecuteNonQuery()

        Catch ex As Exception
            Util.ShowErrorMessage(Application.ProductName, ex)
        Finally
            Me.Cursor = Cursors.Default
            If connection.State = Data.ConnectionState.Open Then
                connection.Close()
            End If

        End Try
    End Sub

    Private Sub ClearForm()
        treeMeta.Nodes.Clear()
        txtMeasures.Clear()
        txtDimensions.Clear()
    End Sub

    Private Sub Disconnect()
        If Not (m_server Is Nothing) Then
            m_server.Disconnect()
        End If
    End Sub 'Disconnect

    '/ <summary>
    '/ Gets the user preferences from the isolated storage
    '/ </summary>
    Sub LoadUserSettings()
        Try
            Dim dom As New XmlDocument()
            Dim s As Stream = Util.GetSettingsStream(Me.Name, FileMode.Open, FileAccess.Read, FileShare.Read)
            dom.Load(s)
            txtServer.Text = dom.SelectSingleNode("//server").InnerText
            txtDatabase.Text = dom.SelectSingleNode("//database").InnerText
        Catch ' no big deal if we fail - this is a convenience feature only
        End Try
    End Sub 'LoadUserSettings

    '/ <summary>
    '/ Saves the user preferences to the isolated storage
    '/ </summary>
    Sub SaveUserSettings()
        Try
            Dim s As Stream = Util.GetSettingsStream(Me.Name, FileMode.Create, FileAccess.ReadWrite, FileShare.None)
            Dim w As New XmlTextWriter(s, System.Text.Encoding.UTF8)
            w.WriteStartElement(Me.Name)
            w.WriteElementString("server", txtServer.Text)
            w.WriteElementString("database", txtDatabase.Text)
            w.WriteEndElement()
            w.Close()
        Catch ' no big deal if we fail - this is a convenience feature only
        End Try
    End Sub 'SaveUserSettings

#Region "Events"
    Private Sub Main_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        LoadUserSettings()
    End Sub
    Private Sub Main_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        Disconnect()
        SaveUserSettings()
    End Sub
    Private Sub txtConnection_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)
        btnConnect.Enabled = txtServer.Text.Trim().Length > 0 And txtDatabase.Text.Trim().Length
    End Sub


    Private Sub cmbCubes_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbCubes.SelectionChangeCommitted
        Me.Cursor = Cursors.WaitCursor
        Try
            ClearForm()
            Dim listItem As ListItem = CType(cmbCubes.SelectedItem, ListItem)
            Dim cube As Cube = CType(listItem.Value, Cube)
            LoadMetadata(cube)

            Dim myStream As New MemoryStream()
            Dim writer As New XmlTextWriter(myStream, New System.Text.UTF8Encoding())

            Utils.Serialize(writer, cube, False)
            System.Diagnostics.Debug.WriteLine(System.Text.Encoding.UTF8.GetString(myStream.GetBuffer()))
        Finally
            Me.Cursor = Cursors.Default
        End Try

    End Sub

    Private Sub treeMeta_ItemDrag(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ItemDragEventArgs) Handles treeMeta.ItemDrag

        Dim node As TreeNode = CType(e.Item, TreeNode)
        If node.Level <> 1 Then
            Return
        End If
        DoDragDrop(e.Item, DragDropEffects.All)
    End Sub


    Private Sub txtMeasures_DragDrop(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles txtMeasures.DragDrop
        Dim node As TreeNode = e.Data.GetData(GetType(TreeNode))

        Dim measure As String = String.Format("[{0}]", CType(node.Tag, Measure).Name)

        txtMeasures.AppendText(IIf(txtMeasures.Text.Trim() = String.Empty, measure, ", " + measure))

    End Sub

    Private Sub txtMeasures_DragOver(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles treeMeta.DragOver, txtMeasures.DragOver
        If Not (e.Data.GetDataPresent(GetType(TreeNode))) Then
            Return
        End If

        Dim node As TreeNode = e.Data.GetData(GetType(TreeNode))

        If TypeOf node.Tag Is Measure Then
            If Not txtMeasures.Text.Contains(CType(node.Tag, Measure).Name) Then
                e.Effect = e.AllowedEffect
            End If
        Else
            e.Effect = DragDropEffects.None
        End If
    End Sub

    Private Sub txtDimension_DragOver(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles txtDimensions.DragOver
        If Not (e.Data.GetDataPresent(GetType(TreeNode))) Then
            Return
        End If

        Dim node As TreeNode = e.Data.GetData(GetType(TreeNode))

        If TypeOf node.Tag Is CubeDimension Then
            If Not txtDimensions.Text.Contains(CType(node.Tag, CubeDimension).Name) Then
                e.Effect = e.AllowedEffect
            End If
        Else
            e.Effect = DragDropEffects.None
        End If
    End Sub

    Private Sub txtDimension_DragDrop(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles txtDimensions.DragDrop
        Dim node As TreeNode = e.Data.GetData(GetType(TreeNode))

        Dim dimension As String = String.Format("[{0}]", CType(node.Tag, CubeDimension).Name)

        txtDimensions.AppendText(IIf(txtDimensions.Text.Trim() = String.Empty, dimension, ", " + dimension))
    End Sub
    Private Sub Query_Changed(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtMeasures.TextChanged, txtDimensions.TextChanged
        GenerateQuery()
    End Sub
    Private Sub btnGenerate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGenerate.Click
        CreateLocalCube()
    End Sub
#End Region


End Class
''' <summary>
''' Helper class to bind a cube details to a combobox
''' </summary>
''' <remarks></remarks>
Public Class ListItem
    Private m_value As Object
    Private m_text As String

    Public Sub New(ByVal itemText As String, ByVal itemValue As Object)
        m_text = itemText
        m_value = itemValue
    End Sub

    Public Property Value() As Object
        Get
            Return m_Value
        End Get
        Set(ByVal Value As Object)
            m_Value = Value
        End Set
    End Property


    Public Property Text() As String
        Get
            Return m_Text
        End Get
        Set(ByVal Value As String)
            m_Text = Value
        End Set
    End Property


    Public Overrides Function ToString() As String
        Return m_Text
    End Function
End Class


